/**
  ******************************************************************************
  * @file    stm32xx_nvm.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   NVM driver
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32xx_nvm_conf.h"


/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
typedef struct{
	eNVM_Operation_t	eOperation;
	uint8_t				*SourceAddress;
	uint8_t				*DestAddress;
	uint16_t			Size;
}sNVM_OperationContext_t;

/* Private macros ------------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile sNVM_OperationContext_t	aNVM_Operation_Queue[NVM_OPERATION_QUEUE_SIZE];
volatile uint8_t		IndexOfNextFreeSlotInQueue;
volatile uint8_t		IndexOfPendingCommandToExecute;
volatile uint8_t		NumberOfPendingCommand;
uint8_t					*pCurrentPointerToFreeShadowBufferLocation;


/* Private function prototypes -----------------------------------------------*/
static void nvm_operation(void);


/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Execute the operation required
  *
  * @note	This API execute the operation (Erase/Write) that is in the queue
  * 		As it always write a 4bytes data, the write operation is done in following steps
  * 		1) Set the read point to the correct 4bytes boundary address
  * 		2) Update the read value with the data to write
  * 		3/ Write the 4 bytes data in the EEPROM
  *
  * @param  None
  *
  * @retval None
  */
static void nvm_operation(void)
{
	uint32_t index;
	uint32_t loop;
	uint32_t data;
	uint32_t *pAddressAligned;
	uint8_t	localIndexOfPendingCommandToExecute;

	localIndexOfPendingCommandToExecute = IndexOfPendingCommandToExecute;

	/*
	 * Calculate the 4 bytes aligned address in EEPROM where the data shall be written
	 */
	pAddressAligned = (uint32_t*)((uint32_t)(aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].DestAddress) & (uint32_t)0xFFFFFFFC);

	/*
	 * Calculate offset where the data shall be written in EEPROM
	 */
	index = ((uint32_t)aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].DestAddress & 0x03);

	/*
	 * Read the data from EEPROM
	 */
	data = *pAddressAligned;

	if(aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].Size >= 4)
	{
		if(aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].eOperation == eNVM_Write)
		{
			for(loop = 0; loop < (4-index); loop++)
			{
				((uint8_t *)&data)[loop + index] = aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].SourceAddress[loop];
			}
		}
		else
		{
			/*
			 * Clear the bytes location to be erased
			 */
			data = ( data & (~((0xFFFFFFFF) << (8*index))));
		}

		aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].Size = aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].Size - 4 + index;
		aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].DestAddress = aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].DestAddress + 4 - index;
		aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].SourceAddress = aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].SourceAddress + 4 - index;

		*pAddressAligned = data;
	}
	else
	{
		for(loop = 0; loop < aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].Size; loop++)
		{
			if(aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].eOperation == eNVM_Write)
			{
				((uint8_t *)&data)[loop] = aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].SourceAddress[loop];
			}
			else
			{
				((uint8_t *)&data)[loop] = 0x00;
			}
		}

		aNVM_Operation_Queue[localIndexOfPendingCommandToExecute].Size = 0;

		*pAddressAligned = data;
	}

	return;
}


/* Public functions ----------------------------------------------------------*/

/**
  * @brief  Initialize the NVM driver
  *
  * @note	This API initializes the NVM pointer used in the driver and enables the NVIC interrupt to schedule consecutive write
  *
  * @param  None
  *
  * @retval None
  */
void NVM_Init(void)
{
	IndexOfNextFreeSlotInQueue = 0;
	IndexOfPendingCommandToExecute = 0;
	NumberOfPendingCommand = 0;

	pCurrentPointerToFreeShadowBufferLocation = (uint8_t *)NVM_BASE_ADDRESS;

	/* Set NVIC priority */
	HAL_NVIC_SetPriority(NVIC_NVM_HANDLER_ID, NVM_IT_PREEMPTPRIORITY, NVM_IT_SUBPRIORITY);

	/* Enable NVIC interrupt */
	HAL_NVIC_EnableIRQ(NVIC_NVM_HANDLER_ID);

	HAL_DATA_EEPROMEx_Unlock();
	__HAL_FLASH_ENABLE_IT(FLASH_IT_EOP);
	HAL_DATA_EEPROMEx_Lock();

	return;
}

/**
  * @brief  Interface to the user to reserve a block in the EEPROM
  *
  * @note
  *
  * @param  NVMAddressBlock: Address reserved to the user
  *
  * @param  BlockSize: Size of block reserved
  *
  * @retval None
  */
void NVM_Register(uint32_t *NVMAddressBlock, uint16_t BlockSize)
{
	NVM_ENTER_CRITICAL_SECTION;
	*NVMAddressBlock = (uint32_t)pCurrentPointerToFreeShadowBufferLocation;
	pCurrentPointerToFreeShadowBufferLocation += BlockSize;
	NVM_EXIT_CRITICAL_SECTION;

	return;
}

/**
  * @brief  Interface to the user to request a Write or Erase operation in the NVM
  *
  * @note	When a write operation is required, the source data shall be kept available as long as the
  * 		NVM driver is running operation (Write or Erase) with the NVM
  *
  * @param  eOperation: Write or Erase operation
  *
  * @param  UserAddress: Source of data to be written (Not used when Erase operation is requested)
  *
  * @param  NVMAddress: NVM destination address
  *
  * @param  Size: Size of data to write/erase
  *
  * @retval None
  */
void NVM_Operation(eNVM_Operation_t eOperation, uint8_t *UserAddress, uint8_t *NVMAddress, uint16_t Size)
{
	NVM_ENTER_CRITICAL_SECTION;

	aNVM_Operation_Queue[IndexOfNextFreeSlotInQueue].eOperation = eOperation;
	aNVM_Operation_Queue[IndexOfNextFreeSlotInQueue].SourceAddress = UserAddress;
	aNVM_Operation_Queue[IndexOfNextFreeSlotInQueue].DestAddress = NVMAddress;
	aNVM_Operation_Queue[IndexOfNextFreeSlotInQueue].Size = Size;

	IndexOfNextFreeSlotInQueue++;
	IndexOfNextFreeSlotInQueue = (IndexOfNextFreeSlotInQueue % NVM_OPERATION_QUEUE_SIZE);
	NumberOfPendingCommand++;

	if(NumberOfPendingCommand == 1)
	{
		NVM_EXIT_CRITICAL_SECTION;

		NVM_HSclkRequest(eNVM_HSclkEnable);

		HAL_DATA_EEPROMEx_Unlock();

		nvm_operation();
	}
	else
	{
		NVM_EXIT_CRITICAL_SECTION;
	}

	return;
}

/**
  * @brief  Interface to the user to read data from the NVM
  *
  * @note  The data read is done asynchronously to the write/erase operation. There is no check of operation in the queue
  * 	   The user shall make sure to not read a data that may be requested to be written in the queue
  *
  * @param  UserAddress: Destination address where to store the data read
  *
  * @param  NVMAddress: Source address in NVM to read
  *
  * @param  Size: Size of data to read
  *
  * @retval None
  */
void NVM_Read(uint8_t *UserAddress, uint8_t *NVMAddress, uint16_t Size)
{
	uint32_t index;

	for(index = 0 ; index < (uint32_t)Size ; index++)
	{
		*(UserAddress + index) = *(NVMAddress + index);
	}

	return;
}

/**
  * @brief  Flash interrupt handler
  *
  * @note  The driver is writing pending data from the queue in the NVM when it is notified the previous operation is completed
  * 	   In the meantime, the application may execute code when the NVM is in a different bank than the user code.
  *
  * @param  None
  *
  * @retval None
  */
void NVM_Flash_Interrupt_handler(void)
{
	__HAL_FLASH_CLEAR_FLAG(FLASH_FLAG_EOP);

	if(aNVM_Operation_Queue[IndexOfPendingCommandToExecute].Size != 0)
	{
		nvm_operation();
	}
	else
	{
		NVM_ENTER_CRITICAL_SECTION;

		NumberOfPendingCommand--;

		IndexOfPendingCommandToExecute++;
		IndexOfPendingCommandToExecute = (IndexOfPendingCommandToExecute % NVM_OPERATION_QUEUE_SIZE);

		if(NumberOfPendingCommand != 0)
		{
			NVM_EXIT_CRITICAL_SECTION;

			nvm_operation();
		}
		else
		{
			HAL_DATA_EEPROMEx_Lock();

			NVM_EXIT_CRITICAL_SECTION;

			NVM_HSclkRequest(eNVM_HSclkDisable);
		}
}

	return;
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
